/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Marco Hutta

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/


#ifndef ScatterWeb_Time
#define ScatterWeb_Time

#include "ScatterWeb.System.h"

// Reads currentTime with disabled interrupts.
#define CURRENT_TIME \
({ \
  extern volatile time_t currentTime; \
  time_t time;  \
  dint(); \
  _NOP(); \
  time = currentTime; \
  eint(); \
  time; \
}) 

// Reads the UINT32 secs from currentTime with disabled interrupts.
#define CURRENT_TIME_SECS \
({ \
  extern volatile time_t currentTime; \
  UINT32 secs;  \
  dint(); \
  _NOP(); \
  secs = currentTime.secs; \
  eint(); \
  secs; \
}) 

// struct of time
typedef struct {
  UINT8 year;
  UINT8 month;
  UINT8 day;
  UINT8 hour;
  UINT8 min;
  UINT8 sec;
} btime_t;

// struct of system time
typedef struct {
  UINT32 secs;
  UINT16 millis;
} time_t;

/*
 * Returns system time
 *  
 * @return bool     True always  
 *   
 */
bool Time_getSystemTime(time_t* time);

/*
 * Sets system time
 * 
 * @return bool     True always  
 *    
 */
bool Time_setSystemTime(time_t* time);

/*
 * Converts system time to time
 * 
 * @return bool     True always  
 *  
 */
bool Time_convertSystemTime(const time_t* src, btime_t* dst);

/*
 * Converts time to system time
 * 
 * @return bool     True always  
 *  
 */
bool Time_convertTime(const btime_t* src, time_t* dst);

/*
 * Writes time
 *  
 * @return bool     True always  
 *   
 */
bool Time_writeTime(fp_char_t writer, btime_t* time);

/*
 * Reads time
 *  
 * @return bool     True always  
 *   
 */
bool Time_readTime(const UINT8* c, btime_t* time);

#endif
